/*
 * Match tag expressions against sets of Debtags items
 *
 * Copyright (C) 2005  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <debtags/Expression.h>
#include <debtags/Serializer.h>

using namespace std;

namespace Tagcoll {

template<>
bool Expression::operator()(const OpSet<Debtags::Tag>& tags) const
{
	Converter<Debtags::Tag, std::string> conv;
	return this->m_impl->eval(conv(tags));
}

template<>
bool Expression::operator()(const OpSet<Debtags::Facet>& tags) const
{
	Converter<Debtags::Facet, std::string> conv;
	return this->m_impl->eval(conv(tags));
}

}

#ifndef INSTANTIATING_TEMPLATES
#include <string>

namespace Tagcoll {
//	template class Expression<Debtags::Tag>;
//	template class Expression<Debtags::Facet>;
}
#endif


#ifdef COMPILE_TESTSUITE

#include <tests/test-utils.h>
#include <debtags/Vocabulary.h>

namespace tut {

using namespace Debtags;
using namespace tut_debtags;

struct debtags_expression_shar {
	TestEnvironment env;
};
TESTGRP(debtags_expression);

template<> template<>
void to::test<1>()
{
	Vocabulary voc(env.voc_fname(), env.vocidx_fname());

	OpSet<Tag> test;
	test += voc.getTag("pizza::margherita");
	test += voc.getTag("pizza::marinara");
	test += voc.getTag("pasta::carbonara");

	ensure_equals(test.size(), 3);

	Expression e1("pizza::margherita");
	gen_ensure(e1(test));

	Expression e2("pizza::margherita && pizza::marinara");
	gen_ensure(e2(test));

	e1 = Expression("!pizza::margherita");
	gen_ensure(!e1(test));

	e1 = Expression("pizza::margherita || sugo");
	gen_ensure(e1(test));

	e1 = Expression("pizza::margherita && !sugo");
	gen_ensure(e1(test));

	e1 = Expression("pizza::margherita && !pizza::marinara");
	gen_ensure(!e1(test));

	e1 = Expression("(pizza::margherita || sugo) && (pizza::marinara && pasta::carbonara)");
	gen_ensure(e1(test));

	e1 = Expression("!(pizza::margherita && sugo) && (pizza::marinara && pasta::carbonara)");
	gen_ensure(e1(test));

	e1 = Expression("pasta::*");
	gen_ensure(e1(test));

	e1 = Expression("*::carbonara");
	gen_ensure(e1(test));

	e1 = Expression("!*::sugo");
	gen_ensure(e1(test));

	e1 = Expression("*::sugo");
	gen_ensure(!e1(test));
}

}

#endif
// vim:set ts=4 sw=4:
