#ifndef DEBTAGS_FILTERS_H
#define DEBTAGS_FILTERS_H

/** \file
 * Collection of filters for Debtags data
 */

/*
 * Copyright (C) 2005  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <debtags/Tag.h>
#include <tagcoll/Consumer.h>
#include <tagcoll/Filter.h>

namespace Debtags
{

/**
 * Convert a collection of ITEMs tagged with Tags to a collection of
 * ITEMs tagged with only the facets.
 */
template<class ITEM>
class TagToFacet : public Tagcoll::Consumer<ITEM, Tag>
{
protected:
	Tagcoll::Consumer<ITEM, Facet>* consumer;

	Tagcoll::OpSet<Facet> strip(const Tagcoll::OpSet<Tag>& tags)
	{
		Tagcoll::OpSet<Debtags::Facet> res;
		for (Tagcoll::OpSet<Tag>::const_iterator i = tags.begin();
				i != tags.end(); i++)
			res += i->facet();
		return res;
	}

	virtual void consumeItemUntagged(const ITEM& item) { consumer->consume(item); }
	virtual void consumeItem(const ITEM& item, const Tagcoll::OpSet<Debtags::Tag>& tags)
	{
		consumer->consume(item, strip(tags));
	}
	virtual void consumeItemsUntagged(const Tagcoll::OpSet<ITEM>& items) { consumer->consume(items); }
	virtual void consumeItems(const Tagcoll::OpSet<ITEM>& items, const Tagcoll::OpSet<Tag>& tags)
	{
		consumer->consume(items, strip(tags));
	}
	
public:
	TagToFacet() : consumer(0) {}
	TagToFacet(Tagcoll::Consumer<ITEM, Facet>& consumer) : consumer(&consumer) {}

	Tagcoll::Consumer<ITEM, Facet>& getConsumer() const { return *consumer; }
	void setConsumer(Tagcoll::Consumer<ITEM, Facet>& consumer) { this->consumer = &consumer; }
};

}

// vim:set ts=4 sw=4:
#endif
