#include "config.h"

#include <debtags/Specials.h>
#include <tagcoll/SmartHierarchy.h>

using namespace std;
using namespace Debtags;
using namespace Tagcoll;

template<class ITEM>
static OpSet<ITEM> getItems(HierarchyNode<ITEM, Facet>* node)
{
	OpSet<ITEM> items = node->getItems();
	
	for (typename HierarchyNode<ITEM, Facet>::iterator i = node->begin();
			i != node->end(); i++)
		items += getItems(*i);

	return items;
}

namespace Debtags {

template<class ITEM>
void Specials<ITEM>::compute(Status* tracker)
{
	if (computed)
		return;

	Facet f;
	SmartHierarchyNode<ITEM, Facet> node(f, coll, 0);

	if (tracker)
		tracker->setTotal(node.size());

	OpSet<Facet> seen;
	for (typename HierarchyNode<ITEM, Facet>::iterator i = node.begin();
			i != node.end(); i++)
	{
		OpSet<ITEM> items = getItems(*i);

		// Find the items in this branch that are not present in
		// any of the previous ones
		if (!seen.empty())
		{
			OpSet<ITEM> newItems;
			for (typename OpSet<ITEM>::const_iterator j = items.begin();
					j != items.end() && newItems.size() <= 101; j++)
			{
				OpSet<Facet> facets = coll.getTags(*j);
				bool isSeen = false;
				for (OpSet<Facet>::const_iterator i = facets.begin(); i != facets.end(); i++)
					if (seen.contains(*i))
					{
						isSeen = true;
						break;
					}
				/*
				TagSet tags = j->tags();
				bool isSeen = false;
				for (TagSet::const_iterator i = tags.begin(); i != tags.end(); i++)
					if (seen.contains(i->facet()))
					{
						isSeen = true;
						break;
					}
				*/
				if (!isSeen)
					newItems += *j;
			}

			if (!newItems.empty() && (maxPerGroup == 0 || newItems.size() <= maxPerGroup))
				specials.insert(make_pair<Facet, OpSet<ITEM> >((*i)->tag(), newItems));
		}

		seen += (*i)->tag();

		if (tracker && !tracker->advance())
		{
			specials.clear();
			return;
		}
	}

	computed = true;
}

};

#ifndef INSTANTIATING_TEMPLATES
#include <string>

namespace Tagcoll {
	template class Specials<std::string>;
}
#endif


#ifdef COMPILE_TESTSUITE

#include <tests/test-utils.h>

namespace tut {

struct debtags_debtagstodo_shar {
};
TESTGRP(debtags_debtagstodo);

template<> template<>
void to::test<1>()
{
	ensure(false);
}

}

#endif
// vim:set ts=4 sw=4:
