#ifndef DEBTAGS_TODO_H
#define DEBTAGS_TODO_H

/*
 * Various classes that can be used to find areas of Debtags that still need
 * work
 *
 * Copyright (C) 2003,2004,2005  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <debtags/Tag.h>
#include <debtags/Filters.h>
#include <debtags/Status.h>
#include <tagcoll/CardinalityStore.h>
#include <map>


namespace Debtags
{

/**
 * Compute "Special-case packages"
 *
 * Compute a list of those packages whose tag set is 'strange' compared to the
 * other packages.
 *
 * The list corresponds to those packages reported by 
 * debtags facetcoll | tagcoll findspecials
 */
template <class ITEM>
class Specials : public Tagcoll::Consumer<ITEM, Tag>
{
protected:
	Tagcoll::CardinalityStore<ITEM, Facet> coll;
	TagToFacet<ITEM> tagStripper;
	unsigned int maxPerGroup;
	bool computed;

	virtual void consumeItemUntagged(const ITEM& item)
	{
		tagStripper.consume(item);
	}
	virtual void consumeItem(const ITEM& item, const Tagcoll::OpSet<Tag>& tags)
	{
		tagStripper.consume(item, tags);
	}
	virtual void consumeItemsUntagged(const Tagcoll::OpSet<ITEM>& items)
	{
		tagStripper.consume(items);
	}
	virtual void consumeItems(const Tagcoll::OpSet<ITEM>& items, const Tagcoll::OpSet<Tag>& tags)
	{
		tagStripper.consume(items, tags);
	}

public:
	Specials(unsigned int maxPerGroup = 0) :
		tagStripper(coll), maxPerGroup(maxPerGroup), computed(false) {}
	virtual ~Specials() {}
	
	bool isComputed() const { return computed; }
	void compute(Status* tracker = 0);
	void clear()
	{
		coll = Tagcoll::CardinalityStore<ITEM, Facet>(); specials.clear();
		computed = false;
	}

	std::map<Facet, Tagcoll::OpSet<ITEM> > specials;
};

}

// vim:set ts=4 sw=4:
#endif
