#include <debtags/Tag.h>
#include <debtags/TagImpl.h>

using namespace std;
using namespace Debtags;

Facet::Facet() throw ()
	: impl(0) {}

Facet::Facet(FacetImpl* impl) throw ()
	: impl(impl)
{
	if (impl)
		impl->ref();
}

Facet::Facet(const Facet& ns) throw ()
{
	if (ns.impl)
		ns.impl->ref();
	impl = ns.impl;
}

Facet::~Facet() throw ()
{   
	if (impl && impl->unref())
		delete impl;
}

Facet& Facet::operator=(const Facet& ns) throw ()
{   
	if (ns.impl)
		ns.impl->ref();  // Do it early to correctly handle the case of x = x;
	if (impl && impl->unref())
		delete impl;
	impl = ns.impl;
	return *this;
}

bool Facet::operator==(const Facet& ns) const throw ()
{   
	if (impl == 0 && ns.impl == 0)
		return true;
	if (impl == 0 || ns.impl == 0)
		return false;
	return name() == ns.name();
}

bool Facet::operator<(const Facet& ns) const throw ()
{   
	if (impl == 0 && ns.impl == 0)
		return false;
	if (impl == 0 && ns.impl != 0)
		return true;
	if (impl != 0 && ns.impl == 0)
		return false;
	return name() < ns.name();
}

Facet::operator bool() const throw ()
{
	return impl != 0;
}

std::string Facet::name() const throw () { return impl->name(); }
std::string Facet::sdesc() const throw () { return impl->sdesc(); }
std::string Facet::ldesc() const throw () { return impl->ldesc(); }
std::string Facet::fulldata() const throw () { return impl->fulldata(); }
std::string Facet::field(const std::string& name) const throw () { return impl->field(name); }

bool Facet::hasTag(const std::string& name) const throw ()
{
	return impl->hasTag(name);
}

Tagcoll::OpSet<Tag> Facet::tags() const throw ()
{
	return impl->tags();
}



Tag::Tag() throw ()
	: impl(0) {}

Tag::Tag(TagImpl* impl) throw ()
	: impl(impl)
{
	if (impl)
		impl->ref();
}

Tag::Tag(const Tag& ns) throw ()
{
	if (ns.impl)
		ns.impl->ref();
	impl = ns.impl;
}

Tag::~Tag() throw ()
{   
	if (impl && impl->unref())
		delete impl;
}

Tag& Tag::operator=(const Tag& ns) throw ()
{   
	if (ns.impl)
		ns.impl->ref();  // Do it early to correctly handle the case of x = x;
	if (impl && impl->unref())
		delete impl;
	impl = ns.impl;
	return *this;
}

bool Tag::operator==(const Tag& ns) const throw ()
{   
	if (impl == 0 && ns.impl == 0)
		return true;
	if (impl == 0 || ns.impl == 0)
		return false;
	return fullname() == ns.fullname();
}

bool Tag::operator<(const Tag& ns) const throw ()
{   
	if (impl == 0 && ns.impl == 0)
		return false;
	if (impl == 0 && ns.impl != 0)
		return true;
	if (impl != 0 && ns.impl == 0)
		return false;
	return fullname() < ns.fullname();
}

Tag::operator bool() const throw ()
{
	return impl != 0;
}

std::string Tag::name() const throw () { return impl->name(); }
std::string Tag::fullname() const throw () { return impl->fullname(); }
std::string Tag::sdesc() const throw () { return impl->sdesc(); }
std::string Tag::ldesc() const throw () { return impl->ldesc(); }
std::string Tag::fulldata() const throw () { return impl->fulldata(); }
std::string Tag::field(const std::string& name) const throw () { return impl->field(name); }

Facet Tag::facet() const throw ()
{
	return Facet(impl->_facet);
}


#ifdef COMPILE_TESTSUITE

#include <tests/test-utils.h>

namespace tut {

struct debtags_tag_shar {
};
TESTGRP(debtags_tag);

template<> template<>
void to::test<1>()
{
	ensure(false);
}

}

#endif

// vim:set ts=3 sw=3:
