#include <debtags/TagImpl.h>
#include <debtags/Tag.h>
#include <debtags/Vocabulary.h>

using namespace std;
using namespace Debtags;

FacetImpl::FacetImpl(const Vocabulary& voc, const char* buf, unsigned int size) throw ()
	: _ref(0), voc(voc)
{
	data.Scan(buf, size);
}
	
bool FacetImpl::hasTag(const std::string& name) const throw ()
{
	return voc.hasTag(data.FindS("Facet") + "::" + name);
}

std::string FacetImpl::name() const throw ()
{
	return data.FindS("Facet");
}

std::string FacetImpl::sdesc() const throw ()
{
	string desc = data.FindS("Description");
	if (!desc.empty())
	{
		unsigned int send = desc.find('\n');
		if (send == string::npos)
			return desc;
		else
			return desc.substr(0, desc.find('\n'));
	}

	return string();
}

std::string FacetImpl::ldesc() const throw ()
{
	string desc = data.FindS("Description");
	if (!desc.empty())
	{
		unsigned int send = desc.find('\n');
		if (send == string::npos)
			return desc;
		else
			return desc.substr(send + 1);
	}

	return string();
}

std::string FacetImpl::fulldata() const throw ()
{
	const char* start;
	const char* stop;
	data.GetSection(start, stop);
	return string(start, stop-start);
}

std::string FacetImpl::field(const std::string& name) const throw()
{
	return data.FindS(name.c_str());
}


Tagcoll::OpSet<Tag> FacetImpl::tags() const throw ()
{
	return voc.getTags(name());
}


TagImpl::TagImpl(FacetImpl* facet, const char* buf, unsigned int size) throw ()
	: _ref(0), _facet(facet)
{
	_facet->ref();
	data.Scan(buf, size);
}

TagImpl::~TagImpl() throw ()
{
	if (_facet->unref())
		delete _facet;
}

std::string TagImpl::name() const throw ()
{
	string fn = fullname();
	unsigned int p = fn.find("::");
	return (p == string::npos) ? fn : fn.substr(p + 2);
}

std::string TagImpl::fullname() const throw ()
{
	return data.FindS("Tag");
}

std::string TagImpl::sdesc() const throw ()
{
	string desc = data.FindS("Description");
	if (!desc.empty())
	{
		unsigned int send = desc.find('\n');
		if (send == string::npos)
			return desc;
		else
			return desc.substr(0, desc.find('\n'));
	}

	return string();
}

std::string TagImpl::ldesc() const throw ()
{
	string desc = data.FindS("Description");
	if (!desc.empty())
	{
		unsigned int send = desc.find('\n');
		if (send == string::npos)
			return desc;
		else
			return desc.substr(send + 1);
	}

	return string();
}

std::string TagImpl::fulldata() const throw ()
{
	const char* start;
	const char* stop;
	data.GetSection(start, stop);
	return string(start, stop-start);
}

std::string TagImpl::field(const std::string& name) const throw()
{
	return data.FindS(name.c_str());
}


#ifdef COMPILE_TESTSUITE

#include <tests/test-utils.h>

namespace tut {

struct debtags_tagimpl_shar {
};
TESTGRP(debtags_tagimpl);

template<> template<>
void to::test<1>()
{
	ensure(false);
}

}

#endif

// vim:set ts=3 sw=3:
