#ifndef DEBTAGS_TAG_IMPL_H
#define DEBTAGS_TAG_IMPL_H

/** \file
 * Shared implementations of facets and tags
 */

/*
 * Copyright (C) 2003,2004,2005  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//#include <debtags/Vocabulary.h>
#include <tagcoll/OpSet.h>
#include <apt-pkg/tagfile.h>
#include <debtags/fdecls.h>

namespace Debtags {

class Vocabulary;

class FacetImpl
{
protected:
	int _ref;
	
	const Vocabulary& voc;
	pkgTagSection data;

public:
	FacetImpl(const Vocabulary& voc, const char* buf, unsigned int size) throw ();

	/// Increment the reference count for this object
	void ref() throw () { ++_ref; }

	/// Decrement the reference count for this object, returning true when it
	/// reaches 0
	bool unref() throw () { return --_ref == 0; }

	bool hasTag(const std::string& name) const throw ();

	std::string name() const throw ();
	std::string sdesc() const throw ();
	std::string ldesc() const throw ();
	std::string fulldata() const throw ();

	/**
	 * Return the content of an arbitrary field
	 */
	std::string field(const std::string& name) const throw();

	Tagcoll::OpSet<Tag> tags() const throw ();

	friend class Facet;
	friend class Tag;
	friend class Vocabulary;
};

class TagImpl
{
protected:
	int _ref;
	
	FacetImpl* _facet;
	pkgTagSection data;

public:
	TagImpl(FacetImpl* facet, const char* buf, unsigned int size) throw ();
	~TagImpl() throw ();

	/// Increment the reference count for this object
	void ref() throw () { ++_ref; }

	/// Decrement the reference count for this object, returning true when it
	/// reaches 0
	bool unref() throw () { return --_ref == 0; }

	std::string name() const throw ();
	std::string fullname() const throw ();
	std::string sdesc() const throw ();
	std::string ldesc() const throw ();
	std::string fulldata() const throw ();

	/**
	 * Return the content of an arbitrary field
	 */
	std::string field(const std::string& name) const throw();

	friend class Tag;
	friend class Vocabulary;
};

};

// vim:set ts=3 sw=3:
#endif
