#ifndef VOCABULARY_H
#define VOCABULARY_H

/** \file
 * Tag vocabulary access
 */

/*
 * Copyright (C) 2003,2004,2005  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tagcoll/Exception.h>
#include <tagcoll/TDBFile.h>

// tdb wants fcntl.h to be included before it (probably a bug in tdb.h)
#include <fcntl.h>
#include <tdb.h>

#include <string>
#include <debtags/fdecls.h>

namespace Debtags
{

class Vocabulary
{
protected:
	std::string filename;
	int fd;
	Tagcoll::TDBFile db;
	char* vocdata;
	off_t vocdata_len;
	
public:
	Vocabulary(const std::string& filename, const std::string& indexname)
		throw (Tagcoll::SystemException);
	~Vocabulary() throw ();

	/**
	 * Check if the vocabulary contains the facet `name'
	 */
	bool hasFacet(const std::string& name) const throw ();

	/**
	 * Check if the vocabulary contains the tag `fullname'
	 */
	bool hasTag(const std::string& fullname) const throw ();

	/**
	 * Return the facet with the given name
	 */
	Facet getFacet(const std::string& name) const throw ();

	/**
	 * Return the tag with the given full name
	 */
	Tag getTag(const std::string& fillname) const throw ();

	/**
	 * Return all the facets in the vocabulary
	 */
	Tagcoll::OpSet<Facet> getFacets() const throw ();

	/**
	 * Return all the tags in the vocabulary
	 */
	Tagcoll::OpSet<Tag> getTags() const throw ();

	/**
	 * Return the tags in the given facet
	 */
	Tagcoll::OpSet<Tag> getTags(const std::string& facetName) const throw ();

#if 0
	/// Get the DerivedTagList with the Equates: expressions read from the vocabulary
	const DerivedTagList& getEquations() const throw () { return equations; }
	
	/// Get a set with all the facets present in the vocabulary that are matched by `filter'
	FacetSet getFacets(const FacetMatcher& filter) const throw () { return getFiltered(filter); }
#endif
};

};

// vim:set ts=4 sw=4:
#endif
