#ifndef DEBTAGS_ENVIRONMENT_H
#define DEBTAGS_ENVIRONMENT_H

#pragma interface

#include <tagcoll/Exception.h>
#include <tagcoll/ParserBase.h>
#include <tagcoll/TagcollConsumer.h>
#include <tagcoll/Serializer.h>
#include <debtags/Consumer.h>
#include <debtags/TagDB.h>

class pkgAcquireStatus;

namespace Debtags
{
class Tag;
class TagSet;
class PackageDB;
class Package;
class PackageMatcher;
class PackageImpl;
class PackageSet;
class Maintainer;
class MaintainerSet;
class Vocabulary;

/**
 * Note: if you're not interested in using the Package support in libdebtags
 * and are only interested in accessing the tag database, don't use
 * Environment::get().tagDB(), but directly use the static methonds in TagDB,
 * like TagDB::outputPatched.  This avoids creating a TagDB object, which would
 * store in memory the Debtags database associated to Package objects that are
 * not used.
 */
class Environment
{
protected:
	static Environment* instance;

#if 0
	virtual void fillPackageData(Debtags::Package& name) const throw () = 0;
#endif

public:
	// editable: if true, allow modifications to metadata (where implemented)
	// facet_only: if true, remove all tags without facets; if false, add a
	//   facet-name tag for each faceted tag
	static void init(bool editable = false, bool facet_only = true) throw (ParserException, SystemException, ConsistencyCheckException);
	static Environment& get() throw () { return *instance; }

	// Check if the tag database has been created (i.e. if debtags update has been run)
	virtual bool hasTagDatabase() throw () = 0;

#if 0
	// Return a PackageSet with all packages
	virtual const PackageSet& packages() const throw () = 0;

	// Return a MaintainerSet with all maintainers
	virtual const MaintainerSet& maintainers() const throw () = 0;
#endif

	// Return the APT Package Database access class
	virtual const PackageDB& packageDB() const throw () = 0;

	// Return the Debtags serializer
	virtual Tagcoll::Serializer<Package, Tag>& serializer() throw () = 0;

	// Return the Debtags Database access class
	virtual const TagDB<Debtags::Package>& tagDB() const throw () = 0;
	virtual TagDB<Debtags::Package>& tagDB() throw () = 0;

	// Return the Debtags Vocabulary access class
	virtual const Vocabulary& vocabulary() const throw () = 0;


	// Get the Package object given the package name
	virtual Package getPackage(const std::string& name) const throw () = 0;

#if 0
	// Get the Maintainer object given the package name
	virtual Maintainer getMaintainer(const std::string& name) const throw () = 0;
#endif


	// Output the whole package list to `cons'
	virtual void output(PackageConsumer& cons) const throw () = 0;

	// Output the filtered package list to `cons'
	virtual void output(PackageConsumer& cons, PackageMatcher& filter) const throw () = 0;

	// Output the whole package list to `cons'
	virtual void output(Tagcoll::TagcollConsumer<Package, Tag>& cons) const throw () = 0;

	// Output the filtered package list to `cons'
	virtual void output(Tagcoll::TagcollConsumer<Package, Tag>& cons, PackageMatcher& filter) const throw () = 0;


	// Output list of packages related to `pivot' to `cons'
	virtual void outputRelated(PackageConsumer& cons, const Package& pivot, int distance = 1) const throw () = 0;

	// Output list of packages related to the specific tagset to `cons'
	virtual void outputRelated(PackageConsumer& cons, const TagSet& ts, int distance = 1) const throw () = 0;

#if 0
	// Update the Debtags database
	virtual void updateDebtagsDatabase() throw (Exception) = 0;

	// Update the Debtags database, tracking status updates
	virtual void updateDebtagsDatabase(pkgAcquireStatus*) throw (Exception) = 0;
#endif

	friend class PackageImpl;
};

};

// vim:set ts=4 sw=4:
#endif
