#pragma implementation

#include "Maintainer.h"
#include "Environment.h"
#include "Package.h"
#include "PackageSet.h"

using namespace std;
using namespace Debtags;

namespace Debtags {

class MaintainerImpl
{
protected:
	int _ref;
	
	std::string _email;
	std::string _fullname;
#if 0
	PackageSet _packages;
#endif

public:
	MaintainerImpl(const std::string& fullname) throw ();

	/// Increment the reference count for this object
	void ref() throw () { ++_ref; }

	/// Decrement the reference count for this object, returning true when it
	/// reaches 0
	bool unref() throw () { return --_ref == 0; }

	friend class Maintainer;
};

};


MaintainerImpl::MaintainerImpl(const std::string& fullname) throw ()
	: _ref(0), _fullname(fullname)
{
	int mend = _fullname.rfind('>');
	int mbeg = _fullname.rfind('<', mend);
	_email = _fullname.substr(mbeg + 1, mend-mbeg-1);
}


Maintainer::Maintainer() throw ()
	: impl(0) {}

Maintainer::Maintainer(const std::string& fullname) throw ()
{
	impl = new MaintainerImpl(fullname);
	impl->ref();
}

Maintainer::Maintainer(const Maintainer& ns) throw ()
{
	if (ns.impl)
		ns.impl->ref();
	impl = ns.impl;
}

Maintainer::~Maintainer() throw ()
{   
	if (impl && impl->unref())
		delete impl;
}

Maintainer& Maintainer::operator=(const Maintainer& ns) throw ()
{   
	if (ns.impl)
		ns.impl->ref();  // Do it early to correctly handle the case of x = x;
	if (impl && impl->unref())
		delete impl;
	impl = ns.impl;
	return *this;
}

bool Maintainer::operator==(const Maintainer& ns) const throw ()
{
	return email() == ns.email();
}

bool Maintainer::operator<(const Maintainer& ns) const throw ()
{   
	return email() < ns.email();
}

Maintainer::operator bool() const throw ()
{
	return impl != 0;
}


const std::string& Maintainer::email() const throw () { return impl->_email; }
const std::string& Maintainer::fullname() const throw () { return impl->_fullname; }

#if 0
void Maintainer::fullname(const std::string& fullname) throw () { impl->_fullname = fullname; }
void Maintainer::addPackage(const Package& pkg) const throw ()
{
	impl->_packages += pkg;
}

const PackageSet& Maintainer::packages() const throw ()
{
	return impl->_packages;
}
#endif

// vim:set ts=3 sw=3:
