# TODO:
#  Limit foundTags to the top-5 (or top-7) by cardinality
#  Also show packages `debtags related the-top-5-or-7-tags`
#  Change 'more like these' in 'like these'
# Allow to increase the distance ("give me more!")


import Debtags
import sys, os

import pygtk
pygtk.require('2.0')
import gtk, gobject

class BrowserEngine(gobject.GObject):
	__gsignals__ = {
		'changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ())
	}

	def __init__(self):
		gobject.GObject.__init__(self)
		self.changed = gobject.signal_new

		self._dt = Debtags.DebtagsString(0)
		self._search = Debtags.SearchHelperString(self._dt)
		self._coll = Debtags.TagCollStringTag()
	
	def search (self, string):
		input = os.popen("apt-cache search " + string, "r");
		self._search.clearStartPackages()
		for i in input:
			pkg = i.split(' - ')[0]
			print "adding " + pkg
			self._search.addStartPackage(pkg)
		print "done adding"

		self._search.clearWanted()
		self._search.clearUnwanted()

		self.update();

	def foundPackages (self):
		return self._search.getFound()

	def foundTags (self):
		return self._search.getPivot()

	def wantTags (self):
		return self._search.getWanted()

	def unwantTags (self):
		return self._search.getUnwanted()
	
#	def getTags (self, package):
#		return self._dt.getTags(package)
	
	def getCardinality (self, tag):
		return self._coll.getCardinality(tag)
#		card = 0
#		for i in self._foundPackages.getIterable():
#			if self._dt.getTags(i).contains(tag):
#				card = card + 1
#		return card

	def addWantedTag (self, tag):
		size = self._search.getWanted().size()
		self._search.addWanted(tag)
		if self._search.getWanted().size() != size:
			self.update();

	def addUnwantedTag (self, tag):
		size = self._search.getUnwanted().size()
		self._search.addUnwanted(tag)
		if self._search.getUnwanted().size() != size:
			self.update();

	def update (self):
		self._search.compute();

		# Build the merged tagset with the tags of the items found
		self._coll = Debtags.TagCollStringTag()
		for i in self._search.getFound().getIterable():
			tags = self._dt.getTags(i)
			self._coll.consume(i, tags)

		self.emit('changed')
	

gobject.type_register(BrowserEngine)

class TagList(gtk.ScrolledWindow):
	def __init__(self, name, engine):
		gtk.ScrolledWindow.__init__(self)
		self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.engine = engine

		self.model = gtk.ListStore(str, str)
		self.list = gtk.TreeView(self.model)
		self.list.set_reorderable(False)

		self.add(self.list)

		cell = gtk.CellRendererText()
		column = gtk.TreeViewColumn(name, cell)
		self.list.append_column(column)
		column.add_attribute(cell, 'text', 1)
		column.set_sort_column_id(0)
		self.list.set_search_column(0)

		self.list.drag_dest_set(gtk.DEST_DEFAULT_ALL, [ ('text/plain', 0, 0) ],
				gtk.gdk.ACTION_COPY)
		self.list.connect("drag-data-received", self.receive_data)

	def receive_data (self, treeview, context, x, y, selection, info, timestamp):
		data = selection.data
		tag = self.engine._dt.vocabulary().getTag(data)
		if tag:
			self.addTag(tag)

	def addTag (self, tag):
		self.model.append([tag.fullname(), tag.sdesc()])


class WantedTagList(TagList):
	def __init__ (self, name, engine):
		TagList.__init__(self, name, engine)
		self.engine.connect("changed", self.update)

	def addTag (self, tag):
		self.engine.addWantedTag(tag)
		
	def update (self, data = None):
		self.model.clear()
		for i in self.engine.wantTags().getIterable():
			self.model.append([i.fullname(), i.sdesc()])

class UnwantedTagList(TagList):
	def __init__ (self, name, engine):
		TagList.__init__(self, name, engine)
		self.engine.connect("changed", self.update)

	def addTag (self, tag):
		self.engine.addUnwantedTag(tag)

	def update (self, data = None):
		self.model.clear()
		for i in self.engine.unwantTags().getIterable():
			self.model.append([i.fullname(), i.sdesc()])
		

class FoundTagList(gtk.ScrolledWindow):
	def __init__(self, name, engine):
		gtk.ScrolledWindow.__init__(self)
		self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

		self.engine = engine
		self.engine.connect("changed", self.update)

		self.model = gtk.ListStore(str, str, int)
		self.list = gtk.TreeView(self.model)

		self.list.set_reorderable(False)

		self.add(self.list)

		cell = gtk.CellRendererText()
		column = gtk.TreeViewColumn(name, cell)
		self.list.append_column(column)
		column.add_attribute(cell, 'text', 1)
		column.set_sort_column_id(1)
		column.set_resizable(True)
		self.list.set_search_column(0)

		cell = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Cardinality", cell)
		self.list.append_column(column)
		column.add_attribute(cell, 'text', 2)
		column.set_sort_column_id(2)
		column.set_resizable(True)

		self.list.drag_source_set(
				gtk.gdk.BUTTON1_MASK, [ ('text/plain', 0, 0) ],
				gtk.gdk.ACTION_COPY)
		self.list.connect("drag_data_get", self.send_data)

	def send_data (self, treeview, context, selection, info, timestamp):
		print "SENDDATA"
		treeselection = treeview.get_selection()
		model, iter = treeselection.get_selected()
		text = model.get_value(iter, 0)
		print "Send " + text
		selection.set('text/plain', 8, text)
	
	def widget (self):
		return self.scroll

	def update (self, data = None):
		self.model.clear()
		cards = {}
		for i in (self.engine.foundTags() - self.engine.wantTags()).getIterable():
			cards[i.fullname()] = (i, self.engine.getCardinality(i))

		tmp = cards.values()
		tmp.sort(lambda x, y: -cmp(x[1], y[1]))
		for i in tmp[0:7]:
			tag, card = i
			self.model.append([tag.fullname(), tag.sdesc(), card])
#			card = self.engine.getCardinality(i)
#			self.model.append([i.fullname(), i.sdesc(), card])
#		for i in (self.engine.foundTags() - self.engine.wantTags()).getIterable():
#			card = self.engine.getCardinality(i)
#			self.model.append([i.fullname(), i.sdesc(), card])

class BrowserWindow:
	def __init__(self):
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.connect("destroy", self.destroy)

		self.engine = BrowserEngine()
		self.engine.connect("changed", self.update)

		self.entry = gtk.Entry()
		self.packageListModel = gtk.ListStore(str)
		self.packageList = gtk.TreeView(self.packageListModel)

		self.tagList = FoundTagList("Tags", self.engine)
		self.wantList = WantedTagList("More like these", self.engine)
		self.nowantList = UnwantedTagList("Not like these", self.engine)

		# Main vbox for the window
		box = gtk.VBox(False)
		self.window.add(box)

		# Create the full-text search bar
		hbox = gtk.HBox(False)
		box.pack_start(hbox, False, False)
		hbox.pack_start(gtk.Label("Search:"), False, False)
		hbox.pack_start(self.entry)
		self.entry.connect("activate", self.searchTextChanged)
		# TODO: add a "go" button

		vpaned = gtk.VPaned()
		box.pack_start(vpaned)

		# Create the tag lists
		hbox = gtk.HBox(False)
		vpaned.pack1(hbox)
		hbox.pack_start(self.tagList)
		hbox.pack_start(self.wantList)
		hbox.pack_start(self.nowantList)

		# Create the package list view
		scroll = gtk.ScrolledWindow()
		scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		scroll.add(self.packageList)
		vpaned.pack2(scroll)

		column = gtk.TreeViewColumn('Package name')
		self.packageList.append_column(column)
		cell = gtk.CellRendererText()
		column.pack_start(cell, True)
		column.add_attribute(cell, 'text', 0)
		column.set_sort_column_id(0)

		self.packageList.set_search_column(0)

		# Add the bottom label
		self.status = gtk.Label("Found:");
		box.pack_start(self.status, False, False)

		self.window.show_all()

	def destroy (self, widget, data=None):
		gtk.main_quit()

	def searchTextChanged (self, widget, entry = False):
		text = self.entry.get_text()
		self.engine.search(text)

	def update (self, data = None):
		self.packageListModel.clear()
		found = self.engine.foundPackages();
		for i in found.getIterable():
			self.packageListModel.append([i])
		self.status.set_text("Found: %i" % found.size())

	def main(self):
		gtk.main()


if __name__ == "__main__":
	base = BrowserWindow()
	base.main()
