/*
 * Test Vocabulary
 *
 * Copyright (C) 2005  Enrico Zini <enrico@debian.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include <test-utils.h>
#include <tagcoll/Exception.h>
#include <tagcoll/StringParserInput.h>
#include <tagcoll/StdioParserInput.h>
#include <debtags/VocabularyMerger.h>
#include <debtags/Vocabulary.h>
#include <debtags/Tag.h>

static const char* init_data0 =
	"Facet: pizza\n"
	"Description: Delicious Italian dish\n\n"
	"Tag: pizza::margherita\n"
	"Description: mozzarella and tomato\n\n"
	"Tag: pizza::marinara\n"
	"Description: tomato and garlic\n\n";

static const char* init_data1 =
	"Tag: pizza::peperoni\n"
	"Description: Tomato, mozzarella and capsicum\n"
	" Foreigners incorrectly call pizza with spicy sausage\n"
	" \"pepperoni\" pizza.  How uncivilised.\n\n"
	"Facet: pizza\n"
	"Description: Very delicious Italian dish\n\n"
	"Facet: pasta\n"
	"Description: Another very delicious Italian dish\n\n"
	"Tag: pasta::carbonara\n"
	"Description: Smoked bacon, eggs and pepper\n"
	" Pepper should be dusted all over the dishes, to make it look\n"
	" like it just came out of a coal mine.  Carbonara means \"to the\n"
	" manner of coal miners\"\n"
	" .\n"
	" Outside of Italy it is common to find distasteful versions\n"
	" with cream, cheese slices and other oddities.\n\n";
	

namespace tut {
using namespace Debtags;
using namespace tut_debtags;

	struct vocabulary_test {
		vocabulary_test() {
			//fprintf(stderr, "Start\n");
			//unlink("test.tdb");
		}
		~vocabulary_test() {
			//fprintf(stderr, "End\n");
			//unlink("test.tdb");
		}
	};
	typedef test_group<vocabulary_test> tg;
	typedef tg::object to;
	tg tdbvocabulary_tg ("Vocabulary");

	template<> template<>
	void to::test<1> ()
	{
		try {
			string inputbuf = init_data0;
			StringParserInput in(inputbuf);
			/*
			fprintf(stderr, "Input: \n---------\n%s\n---------\n", init_data);
			int c;
			while ((c = in.nextChar()) != ParserInput::Eof)
				fputc(c, stderr);
			*/
			VocabularyMerger merger;
			merger.read(in);
			gen_ensure(merger.hasFacet("pizza"));
			gen_ensure(!merger.hasFacet("puzza"));
			gen_ensure(!merger.hasFacet("pizza::margherita"));
			gen_ensure(!merger.hasTag("pizza"));
			gen_ensure(!merger.hasTag("puzza"));
			gen_ensure(merger.hasTag("pizza::margherita"));
			gen_ensure(merger.hasTag("pizza::marinara"));
			gen_ensure(!merger.hasTag("pizza::peperoni"));
			gen_ensure(!merger.hasFacet("pasta"));
			gen_ensure(!merger.hasTag("pasta::carbonara"));

			string inputbuf1 = init_data1;
			StringParserInput in1(inputbuf1);
			merger.read(in1);

			gen_ensure(merger.hasFacet("pizza"));
			gen_ensure(!merger.hasFacet("puzza"));
			gen_ensure(!merger.hasFacet("pizza::margherita"));
			gen_ensure(!merger.hasFacet("pizza::peperoni"));
			gen_ensure(!merger.hasTag("pizza"));
			gen_ensure(!merger.hasTag("puzza"));
			gen_ensure(merger.hasTag("pizza::margherita"));
			gen_ensure(merger.hasTag("pizza::marinara"));
			gen_ensure(merger.hasTag("pizza::peperoni"));
			gen_ensure(merger.hasFacet("pasta"));
			gen_ensure(merger.hasTag("pasta::carbonara"));

			merger.write("testvoc");
			merger.writeIndex("testvoc.idx");
		} catch (Exception& e) {
			fprintf(stderr, "%s: %.*s\n", e.type(), PFSTR(e.desc()));
			throw;
		}
	}

	template<> template<>
	void to::test<2> ()
	{
		try {
			Vocabulary voc("testvoc", "testvoc.idx");

			gen_ensure(voc.hasFacet("pizza"));
			gen_ensure(!voc.hasFacet("puzza"));
			gen_ensure(!voc.hasFacet("pizza::margherita"));
			gen_ensure(!voc.hasFacet("pizza::peperoni"));
			gen_ensure(!voc.hasTag("pizza"));
			gen_ensure(!voc.hasTag("puzza"));
			gen_ensure(voc.hasTag("pizza::margherita"));
			gen_ensure(voc.hasTag("pizza::marinara"));
			gen_ensure(voc.hasTag("pizza::peperoni"));
			gen_ensure(voc.hasFacet("pasta"));
			gen_ensure(voc.hasTag("pasta::carbonara"));

			Facet f = voc.getFacet("puzza");
			gen_ensure(!f);
			f = voc.getFacet("pizza");
			gen_ensure(f);
			gen_ensure(f.name() == "pizza");
			gen_ensure(f.sdesc() == "Very delicious Italian dish");
			gen_ensure(f.ldesc() == "Very delicious Italian dish");
			gen_ensure(f.hasTag("margherita"));
			gen_ensure(f.hasTag("peperoni"));
			gen_ensure(!f.hasTag("pepperoni"));	// Argh!

			Tag t = voc.getTag("pizza::pepperoni");
			gen_ensure(!t);
			t = voc.getTag("pizza::peperoni");
			gen_ensure(t);
			gen_ensure(t.name() == "peperoni");
			gen_ensure(t.fullname() == "pizza::peperoni");
			gen_ensure(t.sdesc() == "Tomato, mozzarella and capsicum");
			gen_ensure(t.ldesc() == " Foreigners incorrectly call pizza with spicy sausage\n \"pepperoni\" pizza.  How uncivilised.");
			gen_ensure(f == t.facet());
			
			OpSet<Tag> tags = voc.getTags("pizza");
			gen_ensure(tags.contains(t));
			gen_ensure(tags.contains(voc.getTag("pizza::margherita")));
			gen_ensure(tags.contains(voc.getTag("pizza::marinara")));
			gen_ensure(!tags.contains(voc.getTag("pizza::pepperoni")));

			unsigned int oldSize = tags.size();
			tags += t;
			gen_ensure(oldSize == tags.size());
			tags += voc.getTag("pizza::margherita");
			gen_ensure(oldSize == tags.size());
			tags += voc.getTag("pizza::peperoni");
			gen_ensure(oldSize == tags.size());

			tags = voc.getTags();
			t = voc.getTag("pasta::carbonara");
			/*
			for (TagSet::const_iterator i = tags.begin(); i != tags.end(); i++)
				fprintf(stderr, "\t%.*s %s with %.*s\n", PFSTR(i->fullname()),
						*i == t ? "match" : "not match", PFSTR(t.fullname()));
			*/
			gen_ensure(tags.contains(voc.getTag("pizza::margherita")));
			ensure(t);
			gen_ensure(tags.contains(t));
		} catch (Exception& e) {
			fprintf(stderr, "%s: %.*s\n", e.type(), PFSTR(e.desc()));
			throw;
		}
	}

	template<> template<>
	void to::test<3> ()
	{
		// Test Tag/Facet relationships
		try {
			StdioParserInput in("vocabulary");
			VocabularyMerger merger;
			merger.read(in);
			merger.write("vocdata");
			merger.writeIndex("vocdata.idx");

			Vocabulary voc("vocdata", "vocdata.idx");
			OpSet<Tag> tags = voc.getTags();
			gen_ensure(tags.size() == 411);
			for (OpSet<Tag>::const_iterator i = tags.begin();
					i != tags.end(); i++)
			{
				//fprintf(stderr, "Facet: %.*s, tag: %.*s\n", PFSTR(i->facet().name()), PFSTR(i->fullname()));
				gen_ensure(i->facet().hasTag(i->name()));
				string fn = i->fullname();
				unsigned int p = fn.find("::");
				if (p == string::npos)
					gen_ensure(i->facet().name() == "legacy");
				else
					gen_ensure(i->facet().name() == fn.substr(0, p));
			}
		} catch (Exception& e) {
			fprintf(stderr, "%s: %.*s\n", e.type(), PFSTR(e.desc()));
			throw;
		}
	}

}

// vim:set ts=4 sw=4:
